package com.haomostudio.SeekTruthBackend.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.haomostudio.SeekTruthBackend.common.Resp;
import com.haomostudio.SeekTruthBackend.service.DeptService;
import com.haomostudio.SeekTruthBackend.service.HmUtils.Tools;
import com.haomostudio.delivery.po.Dept;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "部门表")
@Controller
public class DeptController {

    @Autowired
    DeptService deptService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'部门表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/depts/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createDept(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "did", required = false) String did,
            @RequestParam(value = "dname", required = false) String dname,
            @RequestParam(value = "createTime", required = false) String createTime,
            @RequestParam(value = "lastUpdateTime", required = false) String lastUpdateTime
    ) {
        Dept item = new Dept();
        String id = Tools.getUUID();
        item.setId(id);
        if( did != null ){
            item.setDid(did);
        }
        if( dname != null ){
            item.setDname(dname);
        }
            item.setCreateTime(new Date());
            item.setLastUpdateTime(new Date());
        deptService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return deptService.get(id);
        //return item;
    }

    @ApiOperation(value = "删除'部门表'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/depts/{dept_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteDept(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "dept_id") String id
    ) {
        Dept item = deptService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        deptService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'部门表'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/depts/{dept_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editDept(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "dept_id") String id,
            @RequestParam(value = "did", required = false) String did,
            @RequestParam(value = "dname", required = false) String dname
    ) {
        Dept item = deptService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( did != null ){
            item.setDid(did);
        }
        if( dname != null ){
            item.setDname(dname);
        }
            item.setLastUpdateTime(new Date());
        deptService.update(item);

        return deptService.get(id);
    }

    @ApiOperation(value = "查询'部门表'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/depts/{dept_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getDept(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "dept_id") String id,
            @RequestParam(value = "did", required = false) String did,
            @RequestParam(value = "dname", required = false) String dname
    ) {
        if(request.getMethod().equals("GET")){
            Dept item = deptService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            Dept item = deptService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( did != null ){
                item.setDid(did);
            }
            if( dname != null ){
                item.setDname(dname);
            }
                item.setLastUpdateTime(new Date());
            deptService.update(item);

            return deptService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            Dept item = deptService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            deptService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'部门表'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/depts",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getDepts(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = deptService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return deptService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/depts/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getDeptsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<Dept> list = deptService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = Dept.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'部门表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/depts/create/batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsDepts(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<Dept> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             Dept item = new Dept();
             String id = Tools.getUUID();
                         item.setId(id);
             if(jsonObject.containsKey("did")   ){
            item.setDid(jsonObject.get("did").toString());
        }
        if(jsonObject.containsKey("dname")   ){
            item.setDname(jsonObject.get("dname").toString());
        }
            item.setCreateTime(new Date());
            item.setLastUpdateTime(new Date());
             deptService.create(item);
             lists.add(deptService.get(id));

             });
        return lists;
     }

     @ApiOperation(value = "批量修改'部门表'表中多条信息", notes = "通过post方法请求，传入表中字段的对应信息，数组中每个对象必须有id属性！")
     @RequestMapping(value = "/depts/edit/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
     @ResponseBody
     public Object editsDepts(
              HttpServletRequest request,
              @RequestHeader(value = "X-Auth-Token", required = false) String token,
              @RequestParam(value = "params", required = false) String params
     ) {

          JSONArray jsonArray = JSON.parseArray(params);
          List<Dept> lists  = new ArrayList<>();

          jsonArray.stream().forEach((Object object) -> {

              JSONObject jsonObject = JSONObject.parseObject(object.toString());

              Dept item = new Dept();

              if(jsonObject.containsKey("id")   ){
            item.setId(jsonObject.get("id").toString());
        }
        if(jsonObject.containsKey("did")   ){
            item.setDid(jsonObject.get("did").toString());
        }
        if(jsonObject.containsKey("dname")   ){
            item.setDname(jsonObject.get("dname").toString());
        }
            item.setLastUpdateTime(new Date());
              deptService.update(item);
              lists.add(deptService.get(jsonObject.get("id").toString()));

          });
         return lists;
      }



    @ApiOperation(value = "批量删除'部门表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/depts/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesDept(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 Dept item = deptService.get(id.toString());
                if (null != item) {
                    deptService.delete(id.toString());
                }

            });
            return Resp.succ("delete success");
        }
}
